/**
 * @file __complex.c
 * @author mfh
 * @since 2 June 2005
 *
 * Replaces "double _Complex" for compilers not supporting the C99 standard.
 */
#include <__complex.h>

#ifndef HAVE__COMPLEX
const double_Complex double_Complex_I = {0.0, 1.0};
const double_Complex double_Complex_ZERO = {0.0, 0.0};

double_Complex 
double_Complex_add (const double_Complex a, const double_Complex b)
{
  double_Complex c;
  c.real = a.real + b.real;
  c.imag = a.imag + b.imag;
  return c;
}

double_Complex 
double_Complex_negate (const double_Complex a)
{
  double_Complex c;
  c.real = -(a.real);
  c.imag = -(a.imag);
  return c;
}

double_Complex 
double_Complex_multiply (const double_Complex a, const double_Complex b)
{
  double_Complex c;
  c.real = a.real * b.real - a.imag * b.imag;
  c.imag = a.real * b.imag + a.imag * b.real;
  return c;
}

int 
double_Complex_not_equal (const double_Complex a, const double_Complex b)
{
  return !(a.real == b.real && a.imag == b.imag);  
}

int 
double_Complex_equal (const double_Complex a, const double_Complex b)
{
  return (a.real == b.real && a.imag == b.imag);  
}

double_Complex
new_double_Complex (const double a, const double b)
{
  double_Complex z;
  z.real = a;
  z.imag = b;
  return z;
}

double 
double_Complex_real_part (const double_Complex a)
{
  return a.real;
}

double 
double_Complex_imag_part (const double_Complex a)
{
  return a.imag;
}

double_Complex 
double_Complex_conj (const double_Complex a)
{
  double_Complex c;
  c.real = a.real;
  c.imag = -(a.imag);
  return c;
}
#endif /* HAVE__COMPLEX */

