#ifndef __complex_h
#define __complex_h
/**
 * @file __complex.h
 * @author mfh
 * @since 2 June 2005
 *
 * Replaces "double _Complex" for compilers not supporting the C99 standard.
 */
#include <config.h>


#ifndef HAVE__COMPLEX
#  ifdef HAVE_COMPLEX_H
#    warning "You claim to have a double _Complex datatype, but no complex.h include file.  That means either you are wrong about your claim, or, if you are correct, your compiler is messed up.  In the latter case, you should either use a compiler that fully supports the C99 standard, or write your own copy of complex.h that will work with your compiler.  We can work around this difficulty for now, though, by defining a double_Complex datatype ourselves."
#  endif

typedef struct 
{
  double real;
  double imag;
} double_Complex;

const double_Complex double_Complex_I;
const double_Complex double_Complex_ZERO;

double_Complex 
double_Complex_add (const double_Complex a, const double_Complex b);

double_Complex 
double_Complex_negate (const double_Complex a);

double_Complex 
double_Complex_multiply (const double_Complex a, const double_Complex b);

int 
double_Complex_not_equal (const double_Complex a, const double_Complex b);

int 
double_Complex_equal (const double_Complex a, const double_Complex b);

double_Complex
new_double_Complex (const double a, const double b);

double 
double_Complex_real_part (const double_Complex a);

double 
double_Complex_imag_part (const double_Complex a);

double_Complex 
double_Complex_conj (const double_Complex a);


#else
#  include <complex.h>

typedef double _Complex  double_Complex;

#  define double_Complex_add(a,b)        ((a) + (b))
#  define double_Complex_negate(a)       (-(a))
#  define double_Complex_multiply(a,b)   ((a) * (b))
#  define double_Complex_not_equal(a,b)  ((a) != (b))
#  define double_Complex_equal(a,b)      ((a) == (b))
#  define new_double_Complex(a,b)        ((a) + I*(b))
#  define double_Complex_real_part(a)    (creal(a))
#  define double_Complex_imag_part(a)    (cimag(a))
#  define double_Complex_conj(a)         (conj(a))
#  define double_Complex_I               (I)
#  define double_Complex_ZERO            (0.0 + 0.0*I)

#endif



#endif /* __complex_h */
