#ifndef _enumerations_h
#define _enumerations_h
/**
 * @file enumerations.h
 * @author mfh
 * @since 26 May 2005
 *
 * An include file for all the enums and #defines that are globally useful.
 */

/**
 * Enum for base of array indices:  either zero-based (C-style) or one-based 
 * (Fortran-style).
 */
enum
index_base_t {ZERO = 0, ONE = 1};

/**
 * Symmetry type of the matrix.  Unfortunately MatrixMarket doesn't have a 
 * skew-Hermitian type.
 */
enum
symmetry_type_t 
{ 
  UNSYMMETRIC = 0, SYMMETRIC, SKEW_SYMMETRIC, HERMITIAN 
};

/**
 * For matrices with a type of symmetry:  Where the elements of the matrix 
 * are actually stored: in the lower triangle or the upper triangle.
 */
enum
symmetric_storage_location_t
{
  UPPER_TRIANGLE, LOWER_TRIANGLE
};

/**
 * REAL means that the entries of the matrix are real-valued (IEEE 754 double-precision).
 * COMPLEX means that they are complex-valued (using C99 "double _Complex").
 * PATTERN means that no values are stored.
 */
enum
value_type_t
{
  REAL, COMPLEX, PATTERN
};

#endif /* _enumerations_h */
