/**
 * @file smvm_malloc.c
 * @author mfh
 * @date 20 Oct 2005
 *************************************************************************/
#include <config.h>
#include <smvm_util.h>

/*
 * FIXME: POSIX specs say that systems are not even required to
 * support malloc anymore!!!  They should be in stdlib.h instead.  But
 * some older systems may look for them in malloc.h.  This should be a
 * configuration check.
 */
/* #include <malloc.h> */    /* malloc, realloc */
#include <stdio.h>     /* the usual output stuff */
#include <stdlib.h>    /* exit(), malloc(), realloc() */


/*************************************************************************/
void*
mfh_malloc (size_t size, char* srcfile, int linenum)
{
  void *ptr = malloc(size);

  if ((ptr == NULL) && (size > 0))
    {
      fprintf (stderr, "*** ERROR:  at line %d of %s:  failed to "
	       "malloc %u bytes! ***\n", linenum, srcfile, size);
      smvm_exit (EXIT_FAILURE);
    }
  return ptr;
}


/*************************************************************************/
void*
mfh_calloc (size_t num_elements, size_t element_size, char* srcfile, 
	   int linenum)
{
  void *ptr = calloc (num_elements, element_size);
  /* 
   * If we tried to allocate non-zero amount of memory and it failed,
   * abort with an error message. 
   */
  if ((ptr == NULL) && (num_elements > 0) && (element_size > 0))
    {
      fprintf (stderr, "*** ERROR:  at line %d of %s:  failed to "
	       "calloc %u element array of %u byte elements ***\n", 
	       linenum, srcfile, num_elements, element_size);
      smvm_exit (EXIT_FAILURE);
    }
  return ptr;
}


/*************************************************************************/
void*
mfh_realloc (void* ptr, size_t size, char* srcfile, int linenum)
{
  ptr = realloc (ptr, size);

  if ((ptr == NULL) && (size > 0))
    {
      fprintf (stderr, "ERROR:  at line %d of %s:  failed to reallocate "
	       "%u bytes!\n", linenum, srcfile, size);
      smvm_exit (EXIT_FAILURE);
    }

  return ptr;
}


void
mfh_free (void* ptr, char* srcfile, int linenum)
{
  free (ptr);
}

