/**
 * @file test_split_pathname.c
 * @author mfh 
 * @since 27 Jul 2005
 * @date 27 Jul 2005
 *
 * Driver program for testing split_pathname, in smvm_util.h.  Returns 
 * EXIT_SUCCESS if all tests pass, else prints an error message and returns 
 * EXIT_FAILURE.
 */
#include "../smvm_malloc.h"
#include "../smvm_util.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void
test_split_pathname (const char* const path, 
		     const char* const expected_parentdir, 
		     const char* const expected_namestem, 
		     const char* const expected_extn)
{
  char* parentdir = NULL;
  char* namestem = NULL;
  char* extn = NULL;

  split_pathname (&parentdir, &namestem, &extn, path);
  assert (parentdir != NULL);
  assert (namestem != NULL);
  assert (extn != NULL);
  assert (0 == strcmp (parentdir, expected_parentdir));
  assert (0 == strcmp (namestem, expected_namestem));
  assert (0 == strcmp (extn, expected_extn));

  smvm_free (parentdir);
  smvm_free (namestem);
  smvm_free (extn);
}


int 
main (int argc, char** argv)
{
  printf ("=== test_split_pathname ===\n");

  if (argc >= 5)
    test_split_pathname (argv[1], argv[2], argv[3], argv[4]);
  else
    {
      test_split_pathname ("/usr/lib/libgen.h", "/usr/lib", "libgen", "h");
      test_split_pathname ("foo/bar123_/ugabuga", "foo/bar123_", "ugabuga", "");
    }

  printf ("=== Passed test_split_pathname ===\n");
  return EXIT_SUCCESS;
}

