#ifndef _jad_matrix_h
#define _jad_matrix_h
/**
 * @file jad_matrix.h
 * @author mfh
 * @since 05 Jul 2005
 * @date 05 Jul 2005
 *
 * Jagged diagonal (JAD) format sparse matrix: struct and member functions.
 *
 * @note Original author: Ankit Jain (ankit@berkeley.edu), later revised and 
 * expanded by mfh (mhoemmen@cs.berkeley.edu).
 */
#include <enumerations.h>

struct jad_matrix_t 
{
  int n_jagged_diagonals;

  /** 
   * Stores the nonzero values of the matrix.  It is NULL if value_type == 
   * PATTERN, it holds doubles if value_type == REAL, and it holds 
   * double_Complex if value_type == COMPLEX. 
   */
  void* jad_value;

  int * jad_col_index;
  int* jad_diag_start;
  int* jad_prm_nto;
  double* Py;

  /**
   * Symmetry type of the matrix.
   */
  enum symmetry_type_t symmetry_type;

  /**
   * If the matrix has a kind of symmetry (or skew-symmetry): Where the actual 
   * elements of the matrix are stored: in the lower triangle or the upper 
   * triangle.
   */
  enum symmetric_storage_location_t symmetric_storage_location;

  /** 
   * Indicates the type of the entries in jad_value:  REAL means "double", COMPLEX
   * means "double _Complex", PATTERN means the jad_value array is NULL and contains
   * no entries.
   */ 
  enum value_type_t value_type;
};

struct csr_matrix_t; /* forward declaration */

/**
 * Returns a copy of the given sparse matrix A in jagged diagonal format.
 */
struct jad_matrix_t* 
csr_to_jad (struct csr_matrix_t* A);

/**
 * Deallocates the storage used by the given jad_matrix_t object, and frees
 * the jS pointer.
 */
void 
destroy_jad_matrix (struct jad_matrix_t* jS);

/**
 * Deallocates the storage used by the given jad_matrix_t object, without 
 * actually freeing the jS pointer.
 */
void
dealloc_jad_matrix (struct jad_matrix_t* jS);


#endif /* _jad_matrix_h */
