/**
 * @file mfh_matlib.c
 * @date 11 Jul 2005
 * @since 06/08/04 11:16:16 PDT
 * @author mfh
 * @version 1.2
 *
 * Version 1.1 (mfh 1 July 2004): Moved csc_matrix_t struct definition and 
 * member functions into their own files.
 * Version 1.2 (mfh 26 May 2005): Moved coo_matrix_t and coord_elem_t into
 * their own files.
 * 
 * Adapted by mfh 25 Mar 2005.
 */
#include <config.h>

#include <mfh_matlib.h>
#include <csc_matrix.h>

#include <avltree_intpair.h>
#include <smvm_malloc.h>
#include <smvm_util.h>

#include <string.h> /* strcmp */




/*======================================================================*/
void
ones (double* A, const int m, const int n)
{
  const int L = m*n;
  int i;
  for (i = 0; i < L; i++)
	A[i] = 1.0;
}


/*======================================================================*/
void
zeros (double* A, const int m, const int n)
{
  const int L = m*n;
  int i;
  for (i = 0; i < L; i++)
	A[i] = 0.0;
}


/*======================================================================*/
void
unit_vector (double* v, const int n, const int which)
{
  int i;
  if (which < 0 || which >= n)
	{
	  fprintf(stderr,"*** unit_vector: which=%d out of valid range [0,%d) ***\n", which, n);
	  die (-1);
	}
  for (i = 0; i < n; i++) v[i] = 0.0;

  v[which] = 1.0;
}


/*======================================================================*/
void
print_dense_vector (FILE* out, const double* x, const int length)
{
  int i;
  for (i = 0; i < length; i++)
	fprintf (out, "%E\n", x[i]);
}


/*======================================================================*/
void
print_dense_matrix (FILE* out, const double* A, const int m, const int n, 
					const char* trans)
{
  int i, j;

  WITH_DEBUG2(fprintf(stderr, "=== print_dense_matrix ==="));

  if (0 == strcmp (trans, "T"))
    {
      /* 
       * Lazy solution:  transpose the matrix and print the transposed 
       * version.  It's a human-readable-print routine, so the matrix 
       * probably isn't very big anyway.
       */
      double* B = transpose_dense_col_oriented_matrix (A, m, n);
      print_dense_matrix (out, B, n, m, "N");
      smvm_free (B);
    }
  else
    {
      for (i = 0; i < m; i++) /* for each row i */
	{
	  fprintf (out, "[");
	  if (n > 0)
	    {
	      for (j = 0; j < n - 1; j++) /* for each entry in the row */
		fprintf (out, "%E ", A[i + m*j]);

	      fprintf (out, "%E", A[i + m*j]);
	    }
	  fprintf (out, "]\n");
	}
    }

  WITH_DEBUG2(fprintf(stderr, "=== Done with print_dense_matrix ==="));
}


/*======================================================================*/
double* 
transpose_dense_col_oriented_matrix (const double* A, 
				     const int m, const int n)
{
  double* B = smvm_malloc (n * m * sizeof (double));
  int i, j;

  for (i = 0; i < m; i++)
    for (j = 0; j < n; j++)
      B[j + n*i] = A[i + m*j];

  return B;
}


