#ifndef _mfh_matlib_h
#define _mfh_matlib_h
/**
 * @file mfh_matlib.h
 * @date 28 May 2005
 * @since 06/08/04 11:05:44 PDT
 * @author mfh
 * @version 1.2
 * 
 * Some basic sparse and dense matrix manipulation functions.
 *
 * Version 1.1 (mfh 1 July 2004): Moved csc_matrix_t struct definition and 
 * member functions into their own files.
 * Version 1.2 (mfh 26 May 2005): Moved coo_matrix_t and coord_elem_t into
 * their own files.
 */

#include <stdio.h> /* FILE */


/**
 * Prints the given dense vector x[0:length-1] to the given output stream.
 */
void
print_dense_vector (FILE* out, const double* x, const int length);


/**
 * Prints the given dense column-oriented m x n matrix to the given output 
 * stream.
 *
 * @param out [OUT]   Output stream
 * @param A   [IN]    Dense column-oriented m x n matrix
 * @param m   [IN]    Number of rows in A
 * @param n   [IN]    Number of columns in A
 * @param trans [IN]  If "T", prints the transpose of the input matrix A.
 *                    The order of the m and n parameters should not be 
 *                    changed.  This option can be used to print a row-
 *                    oriented matrix.
 */
void
print_dense_matrix (FILE* out, const double* A, const int m, const int n, 
					const char* trans);


/** 
 * Equivalent to Matlab's ``A = ones (m,n)''.  Treats A as a flat (1-D) 
 * array; it may be either column-oriented or row-oriented.
 */
void
ones (double* A, const int m, const int n);


/** 
 * Equivalent to Matlab's ``A = zeros (m,n)''.  Treats A as a flat (1-D) 
 * array; it may be either column-oriented or row-oriented.
 */
void
zeros (double* A, const int m, const int n);


/**
 * Fills v[0:n-1] with the which-th unit vector.
 */
void
unit_vector (double* v, const int n, const int which);


/**
 * Returns the transpose of the given column-oriented dense matrix.
 */
double* 
transpose_dense_col_oriented_matrix (const double* A, 
				     const int m, const int n);


#endif /* _mfh_matlib_h */
