/**
 * @file sparse_matrix.c
 * @author mfh
 * @since 26 May 2005
 * @date 11 Jul 2005
 *
 * A wrapper representation of a general sparse matrix.
 */
#include <config.h>
#include <sparse_matrix.h>
#include <bcoo_matrix.h>
#include <bcsr_matrix.h>
#include <coo_matrix.h>
#include <csc_matrix.h>
#include <csr_matrix.h>
#include <jad_matrix.h>

#include <smvm_malloc.h>
#include <smvm_util.h>


/**
 * Returns nonzero if and only if "format" is one of the supported sparse matrix storage formats.
 */
static int
sparse_matrix_supported_storage_format_p (enum sparse_matrix_storage_format_t format)
{
  if (format == CSC || format == CSR || format == COO || format == BCOO || format == BCSR || format == JAD)
    return 1;
  else 
    return 0;
}

struct sparse_matrix_t*
create_sparse_matrix (enum sparse_matrix_storage_format_t format, void* repr)
{
  struct sparse_matrix_t* A = NULL;

  if (! sparse_matrix_supported_storage_format_p (format))
    return NULL;
  
  A = smvm_malloc (sizeof (struct sparse_matrix_t));
  A->format = format;
  A->repr = repr;
  return A;
}

int
destroy_sparse_matrix (struct sparse_matrix_t* A)
{
  if (A != NULL)
    {
      if (A->format == CSC)
	destroy_csc_matrix (A->repr);
      else if (A->format == CSR)
	destroy_csr_matrix (A->repr);
      else if (A->format == COO)
	destroy_coo_matrix (A->repr);
      else if (A->format == BCOO)
	destroy_bcoo_matrix (A->repr);
      else if (A->format == BCSR)
	destroy_bcsr_matrix (A->repr);
      else if (A->format == JAD)
	destroy_jad_matrix (A->repr);
      else
	{
	  fprintf (stderr, "*** destroy_sparse_matrix: invalid matrix storage format %d ***\n", A->format);
	  return -1; 
	}
      A->repr = NULL;
      smvm_free (A);
    }
  else
    {
      fprintf (stderr, "*** destroy_sparse_matrix: input is NULL! ***\n");
      return -1;
    }
  return 0;
}


const char* const
sparse_matrix_format_string (struct sparse_matrix_t* A)
{
  enum sparse_matrix_storage_format_t format = A->format;

  if (format == CSC)
    return "CSC";
  else if (format == CSR)
    return "CSR";
  else if (format == COO)
    return "COO";
  else if (format == BCOO)
    return "BCOO";
  else if (format == BCSR)
    return "BCSR";
  else if (format == JAD)
    return "JAD";
  else 
    return "unknown";
}
