#ifndef _sparse_matrix_h
#define _sparse_matrix_h
/**
 * @file sparse_matrix.h
 * @author mfh
 * @since 26 May 2005
 * @date 11 Jul 2005
 *
 * A wrapper representation of a general sparse matrix.
 */

/**
 * Enum indicating the internal storage format of the sparse matrix.
 */
enum
sparse_matrix_storage_format_t
{
	COO, CSC, CSR, BCOO, BCSR, JAD
};

/**
 * The supported file formats for sparse matrices.
 */
enum
sparse_matrix_file_format_t
{
	HARWELL_BOEING, MATRIX_MARKET, MATLAB
};

/**
 * Wrapper struct for a sparse matrix.  
 */
struct
sparse_matrix_t
{
  /** The internal storage format of the sparse matrix. */
  enum sparse_matrix_storage_format_t format;

  /** Pointer to the internal representation of the sparse matrix.  If format==CSC, 
   * this is a "struct csc_matrix_t"; if format==COO, this is a "struct coo_matrix_t". */
  void* repr;
};

/**
 * Allocates a wrapper object with shallow copies of the given format and representation.
 */
struct sparse_matrix_t*
create_sparse_matrix (enum sparse_matrix_storage_format_t format, void* repr);

/** 
 * If no error, deallocates the internal representation of the given sparse
 * matrix, as well as the sparse_matrix_t struct itself, and returns zero.  If
 * error, returns nonzero without trying to deallocate anything.
 *
 * @param A [IN/OUT]   Pointer to sparse matrix object
 *
 * @return Zero if no error, else nonzero.
 */
int 
destroy_sparse_matrix (struct sparse_matrix_t* A);

/**
 * Returns a string version of the internal storage format of the given
 * sparse matrix.
 */
const char* const 
sparse_matrix_format_string (struct sparse_matrix_t* A);


#endif /* _sparse_matrix_h */
