#ifndef _sparse_matrix_ops_h
#define _sparse_matrix_ops_h
/**
 * @file sparse_matrix_ops.h
 * @author mfh
 * @since 26 May 2005
 *
 * Operations on general sparse matrices.
 */
#include <sparse_matrix.h>


/**
 * Saves the sparse matrix A to the given file, using the given file format.
 *
 * @param filename [IN]  Name of the file to which to save the matrix
 * @param A [IN]         The matrix to save
 * @param file_format [IN]  Output file format
 */
int
save_sparse_matrix (const char* const filename, 
		    struct sparse_matrix_t* A, 
		    enum sparse_matrix_file_format_t file_format);


/**
 * Loads a sparse matrix from the given file, with the assumption that 
 * the file is in the format file_format.
 *
 * @param file_format [IN]      Input file format 
 * @param matrix_filename [IN]  Name of the file from which to load the matrix
 *
 * @return If no error, the sparse matrix; else NULL.
 */
struct sparse_matrix_t*
load_sparse_matrix (enum sparse_matrix_file_format_t file_format, 
		    const char *const matrix_filename);

/**
 * If A is stored in a space-saving symmetric format, expands A in place so 
 * that all the nonzero elements are stored explicitly.
 *
 * @param A [IN/OUT]  Pointer to sparse matrix object
 *
 * @return Zero if successful, else nonzero.
 */
int
sparse_matrix_expand_symmetric_storage (struct sparse_matrix_t* A);

/**
 * Converts the sparse matrix A from its current format into the new format
 * output_format.
 *
 * @param A [IN/OUT]   Sparse matrix to convert
 * @param output_format [IN]  New storage format
 *
 * @return Zero if successful, else nonzero.
 */
int
sparse_matrix_convert (struct sparse_matrix_t* A, 
		       enum sparse_matrix_storage_format_t output_format);

/**
 * Returns 0 if the given sparse matrix has a valid representation, else 
 * returns nonzero.
 */
int
valid_sparse_matrix (struct sparse_matrix_t* A);

enum sparse_matrix_storage_format_t 
sparse_matrix_storage_format_string_to_enum (const char* const s);

enum sparse_matrix_file_format_t
sparse_matrix_file_format_string_to_enum (const char* const s);

#endif /* _sparse_matrix_ops_h */
