#ifndef _block_size_parser_h
#define _block_size_parser_h
/**
 * @file block_size_parser.h
 * @author mfh
 * @since 2004 Nov 28
 * @date 2004 Nov 28
 *
 * Functions for parsing a list of triples:  r,c,frequency  in CSV format.  
 * These represent optimal block dimensions and the number of times that 
 * pair of dimensions appears.
 */

/**
 * Represents a single data point:  Optimal block dimensions r and c for a 
 * particular matrix in the test set, and the number of times that (r,c) pair
 * comes up in the collection of test matrices among the various machines 
 * tested.
 */
struct block_size_data_t
{
  int r, c, frequency;
};

/**
 * Parses file of block size frequency data, and constructs list of data.
 *
 * @param dataarray [OUT]  Pointer to array of block_size_data_t elements
 *                         extracted from the data file.
 * @param dataarraylen [OUT]  Pointer to number of elements extracted from
 *                            the data file.
 * @param filename [IN]    Name of the data file.
 * 
 * @return Nonzero if error; else zero.
 */
int
parse_block_sizes_file (struct block_size_data_t **dataarray, 
			int *dataarraylen, const char *filename);

#endif /* _block_size_parser_h */
