/**
 * @file hpcc.c
 * @author mfh
 * @date 2004 Nov 21
 */

#include "config.h"
#include "hpcc.h"
#include "smvm_benchmark_results.h"
#include <smvm_malloc.h>



struct my_HPCC_Params
{
#ifdef SPMV_STANDALONE
  int Failure;
  int HPLMaxProcMem;
  double fill;
  struct SMVM_benchmark_results *results;
  char *outFname;
#else
  /** 
   * Pointer to the HPCC_Params object, which has IN/OUT parameters
   * for the benchmark.  The object is owned externally, so we are 
   * not responsible for allocation or deallocation; we assume that
   * it arrives allocated and initialized to the correct values.
   */
  HPCC_Params* params; 
#endif /* SPMV_STANDALONE */
};



void
my_HPCC_Params__report_failure (struct my_HPCC_Params* p, const int failure)
{
#ifdef SPMV_STANDALONE
  p->Failure = failure;
#else
  p->params->Failure = failure;
#endif /* SPMV_STANDALONE */
}

void 
my_HPCC_Params__set_HPLMaxProcMem (struct my_HPCC_Params* p, const int HPLMaxProcMem)
{
#ifdef SPMV_STANDALONE
  p->HPLMaxProcMem = HPLMaxProcMem;
#else
  p->params->HPLMaxProcMem = HPLMaxProcMem;
#endif /* SPMV_STANDALONE */
}


int 
my_HPCC_Params__get_HPLMaxProcMem (struct my_HPCC_Params* p)
{
#ifdef SPMV_STANDALONE
  return p->HPLMaxProcMem;
#else
  return p->params->HPLMaxProcMem;
#endif /* SPMV_STANDALONE */
}


double 
my_HPCC_Params__get_fill (struct my_HPCC_Params* p)
{
#ifdef SPMV_STANDALONE
  return p->fill;
#else
  return p->params->fill;
#endif /* SPMV_STANDALONE */
}


void
my_HPCC_Params__copy_benchmark_results (struct my_HPCC_Params* p, 
					struct SMVM_benchmark_results* results)
{
#ifdef SPMV_STANDALONE
  p->results = smvm_copy_benchmark_results (results);
#else
  p->params->results = smvm_copy_benchmark_results (results);
#endif
}

void
my_HPCC_Params__set_benchmark_results (struct my_HPCC_Params* p,
				       struct SMVM_benchmark_results* results)
{
#ifdef SPMV_STANDALONE
  p->results = results;
#else
  p->params->results = results;
#endif
}


const char* const 
my_HPCC_Params__get_output_filename (const struct my_HPCC_Params* p)
{
#ifdef SPMV_STANDALONE
  return p->outFname;
#else
  return p->params->outFname;
#endif
}


void
my_HPCC_Params__set_output_filename (struct my_HPCC_Params* p, 
				     char* outfilename)
{
#ifdef SPMV_STANDALONE
  p->outFname = outfilename;
#else
  p->params->outFname = outfilename;
#endif
}



void
my_HPCC_Params__destroy (struct my_HPCC_Params* p)
{
  /* 
   * If we are in standalone mode, then p has ownership of the benchmark
   * results object.  Otherwise, the HPCC suite owns the benchmark results.
   */
#ifdef SPMV_STANDALONE
  smvm_free_benchmark_results (p->results);
#endif

  smvm_free (p);
}


#ifdef SPMV_STANDALONE
struct my_HPCC_Params*
my_HPCC_Params__construct_standalone (const int maxmem, const double fill, char* outfilename)
{
  struct my_HPCC_Params *p = smvm_malloc (sizeof (struct my_HPCC_Params));

  p->Failure = 0;
  p->HPLMaxProcMem = maxmem;
  p->fill = fill;
  p->results = NULL;
  p->outFname = outfilename;

  return p;
}
#endif /* SPMV_STANDALONE */
